
;/****************************************************************************
;* DESCRIPTION: System definitions.
;*****************************************************************************/ 

	ERRORLEVEL -302 ;remove message about using proper bank

	LIST  P=16F677
	INCLUDE <P16F677.INC>
	__CONFIG _BOR_OFF & _FCMEN_OFF & _IESO_OFF & _CPD_OFF & _CP_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTOSCIO

;/****************************************************************************
;* DESCRIPTION: Program variables and constant definitions.
;*****************************************************************************/ 

temper		equ 0x20						; Store temperature
units		equ 0x24						; units
tens		equ	0x25						; tens
hundreds	equ 0x26						; hundreds
aux			equ	0x27						; GENERAL aux. REGISTER
tempo0		equ	0x28
tempo1		equ	0x29
flags		equ	0x2B						; General use flags
timer		equ	0x2D						; Timer register
buz_file	equ 0x2E						; Buzzer file
buz_length	equ 0x2F						; Buzzer length
leds_s		equ	0x30						; Leds state
k_pressed	equ	0x31						; Keys pressed state
Key_deb		equ	0x32						; Keys debounce flag
k_not_pressed equ	0x33					; Key off flag
Led_blink	equ	0x34						; Led blink time - for debounce
aux_file	equ	0x35						; timing auxiliar file

sw_flag		equ	0x35						; For keys routines
fastcount	equ	0x36						; For keys routines

#define		LEDS_ON				flags,0		; Flag to see if the leds should be on

; Leds
#define		LED_CMN				PORTC,2		; Leds Anodes

#define		LED_UP				PORTA,4		; Led UP
#define		LED_timer			PORTB,6		; Led timer
#define		LED_TEMP			PORTB,5		; Led TEMP
#define		LED_DWN				PORTA,2		; Led DWN
#define		LED_LIGHT			PORTB,4		; Led LIGHT
#define		LED_TOASTER			PORTA,5		; Led TOASTER
#define		LED_ONOFF			PORTC,6		; Led ONOFF
#define		TRIS_LED_ONOFF		TRISC,6		; Led ONOFF TRIS

; Displays
#define		DISP1				PORTC,3		; Anode display DIS0
#define		DISP2				PORTC,4		; Anode display DIS1
#define		DISP3				PORTC,5		; Anode display DIS2

; Keys
#define		KEY_UP				PORTB,7		; KEY_UP
#define		KEY_timer			PORTB,6		; KEY_timer
#define		KEY_ONOFF			PORTB,5		; KEY_ONOFF
#define		KEY_LIGHT			PORTB,4		; KEY_LIGHT
#define		KEY_TOASTER			PORTA,5		; KEY_TOASTER
#define		KEY_TEMP			PORTA,4		; KEY_TEMP
#define		KEY_DWN				PORTA,2		; KEY_DWN

; Buzzer
#define		buz					PORTC,0		; Buzzer

;/****************************************************************************
;* DESCRIPTION: Start of program.
;***************************************************************************** 
	ORG		0x0000
	goto	set_up

;/****************************************************************************
;* DESCRIPTION: Decodes value in W register to display segments for PORTA.
;* return:      result in W
;*****************************************************************************/ 

decdisp_a:
	addwf   PCL,F		; compute the jump value
						;	|7|6|5|4|3|2|1|0| -	Bits PORTA
						;	|-|-|B|E|-|A|-|-|
	retlw	B'00000000'	;	|-|-|B|E|-|A|-|-| -	0
	retlw	B'00010100'	;	|-|-|B|X|-|X|-|-| -	1
	retlw	B'00000000'	;	|-|-|B|E|-|A|-|-| -	2
	retlw	B'00010000'	;	|-|-|B|X|-|A|-|-| -	3
	retlw	B'00010100'	;	|-|-|B|X|-|X|-|-| -	4
	retlw	B'00110000'	;	|-|-|X|X|-|A|-|-| -	5
	retlw	B'00100000'	;	|-|-|X|E|-|A|-|-| -	6
	retlw	B'00010000'	;	|-|-|B|X|-|A|-|-| -	7
	retlw	B'00000000'	;	|-|-|B|E|-|A|-|-| -	8
	retlw	B'00010000'	;	|-|-|B|X|-|A|-|-| -	9


;/****************************************************************************
;* DESCRIPTION: Decodes value in W register to display segments, for PORTB.
;* return:      result in W
;*****************************************************************************/ 

decdisp_b:
	addwf   PCL,F		; compute the jump value
						;	|7|6|5|4|3|2|1|0| -	Bits PORTB
						;	|C|F|D|G|-|-|-|-| -
	retlw	B'00010000'	;	|C|F|D|X|-|-|-|-| -	0
	retlw	B'01110000'	;	|C|X|X|X|-|-|-|-| -	1
	retlw	B'11000000'	;	|X|X|D|G|-|-|-|-| -	2
	retlw	B'01000000'	;	|C|X|D|G|-|-|-|-| -	3
	retlw	B'00100000'	;	|C|F|X|G|-|-|-|-| -	4
	retlw	B'00000000'	;	|C|F|D|G|-|-|-|-| -	5
	retlw	B'00000000'	;	|C|F|D|G|-|-|-|-| -	6
	retlw	B'01110000'	;	|C|X|X|X|-|-|-|-| -	7
	retlw	B'00000000'	;	|C|F|D|G|-|-|-|-| -	8
	retlw	B'00000000'	;	|C|F|D|G|-|-|-|-| -	9

;/****************************************************************************
;* DESCRIPTION: Program Initialization
;*****************************************************************************/ 
set_up:
	bcf		STATUS,RP0		; RP0 = 0
	bsf		STATUS,RP1		; RP1 = 1 -> Bank 2 (I need it to switch to bank2)
	clrf	ANSEL			; digital I/O
	clrf	ANSELH			;	"	"	"
	movlw	B'00000000'		; To disable comparators
	movwf	CM1CON0			; disable comparator
	movlw	B'00000000'		; Not necessary?
	movwf	CM2CON0			; disable 2nd comparator
	movlw	B'00000000'		; Not necessary?
	movwf	CM2CON1			; disable 3rd comparator
	bsf		STATUS,RP0		; RP0 = 1
	bcf		STATUS,RP1		; RP1 = 0 -> Bank 1 ( need it to came back from bank2)
	movlw	B'00001000'		; RA0,RA1,RA2,RA4,RA5,RA6 and RA7 output - RA3 input
	movwf	TRISA			; Set it
	movlw	B'00000000'		; All PORTB output
	movwf	TRISB			; Set it
	movlw	B'10000010'		; RC0,RC2,RC3,RC4,RC5,RC6 output - RC1 and RC7 inputs
	movwf	TRISC			; Set it
	bcf		STATUS,RP0		; RP0 = 0 (removed the bcf STATUS,RP1)
	bcf		SSPCON,SSPEN	; Disable Synchronous Serial Port
    bsf		STATUS,RP0		; RP0 = 1 goto Bank 1
    bcf		OPTION_REG,NOT_RABPU ; Activate pull-up resistors - Clear RABPU flag
    movlw	B'00110100'		; 1 = enable pull-up - 0 disable it.
    movwf	WPUA			; Enable RA2, RA4 and RA5 pull-up internal resistors
    bcf		STATUS,RP0		; RP0 = 0 -> Bank 2
	bsf		STATUS,RP1		; RP1 = 1 -> "   " (Again, I need it to go to bank2)
    movlw	B'11110000'		; 1 = enable pull-up - 0 disable it.
    movwf	WPUB			; Enable RB4, RB5, RB6 and RB7 pull-up internal resistors
	bcf		STATUS,RP0		; RP0 = 0
	bcf		STATUS,RP1		; RP1 = 0 -> Back to Bank 0 (Again, I need it to came back from bank2)

	; Turn off all displays and leds
	bsf		LED_CMN			; Leds off
	bsf		DISP1			; Display1 off
	bsf		DISP2			; Display2 off
	bsf		DISP3			; Display3 off
	movlw	D'180'			; Init temperature with 180 degrees Celsius
	movwf	temper			; Put it in temper file
	movlw	D'40'			; Start timer with 40 minutes
	movwf	timer			; Put it in timer file
	clrf	units			; Clear dislay's files
	clrf	tens			;
	clrf	hundreds		;
	movlw	D'255'
	movwf	buz_length		; Buzzer initial duration
	movlw	B'11110001'		; Leds initial state (led onoff blinking, up, down and temperature on)
	movwf	leds_s			; Set leds state
	clrf	k_pressed		; Zero keys pressed state
	clrf	Key_deb			; Zero Keys debounce flag
	clrf	k_not_pressed	; Set Key off flag (all keys not pressed)
	movlw	D'255'			; Value to preload to
	movwf	Led_blink		; Led blink time control
    goto    Main

;/****************************************************************************
;* DESCRIPTION: binary_to_bcd - 8-bits
;* INPUT:		temper   - 8-bit binary number
;* return:      hundreds - the hundreds digit of the BCD conversion
;*				tens     - the tens digits of the BCD conversion
;*				units    - the ones digits of the BCD conversion
;*****************************************************************************/ 
bin2bcd:

	movwf	aux				; save the value to convert in aux
	clrf	units			;
	clrf	tens			;
	clrf	hundreds		; RESET variables
	movf	aux,F			;
	btfsc	STATUS,Z		; Is the value to convert = 0?
	return					; Yes - Return
							; No
	incf	units,F			; Increment unit
	movf	units,W			;
	xorlw	0X0A			;
	btfss	STATUS,Z		; unit = 10d ?
	goto	$+3				; No
							; yes
	clrf	units			; Reset unit
	incf	tens,F			; Increment tens
	movf	tens,W			;
	xorlw	0X0A			;
	btfss	STATUS,Z		; Tens = 10d ?
	goto	$+3				; No
							; Yes
	clrf	tens			; Reset tens
	incf	hundreds,F		; Increment hundreds
	decfsz	aux,F			; End of convertion ?
	goto	$-.14			; No - go back to continue the convertion
	return					; Yes

;/****************************************************************************
;* DESCRIPTION: House keeping and delay routine.
;*****************************************************************************/ 
DelayAA

	movlw	D'255'			; This delay increase the on-time for each segment
	movwf	aux_file		; to improve the brightness
	decfsz	aux_file,F		; 
	goto	$-1    
	retlw	00

;/****************************************************************************
;* DESCRIPTION: House keeping and delay routine.
;*****************************************************************************/ 
H_keeping:

    ; Leds:
    ;   7       6       5       4       3       2       1       0
    ; blink   onoff     up     down   light    toast   time    temp

    btfss	leds_s,7		; Is led blink enabled?
    goto	contled			; No, it's not enabled
    btfsc	leds_s,6		; Is led on/off on?
    goto	$+3
    bcf		LED_ONOFF		; No - it's activated directly, not using LED_CMN
    goto	$+2
    bsf		LED_ONOFF		; Yes - it's activated directly, not using LED_CMN

    decfsz	Led_blink,F		; Blink control = 0?
    goto	contled			; No
    movlw	D'255'			; Value to preload to
    movwf	Led_blink		; Led blink time control
    btfsc	leds_s,6		; Is led on/off on?
    goto	setledoff		; Yes, turn it off.
    bsf		leds_s,6		; No, turn it on
    goto	contled			; Continue
setledoff:
    bcf		leds_s,6		; Led on/off in off
contled:
    btfss	LEDS_ON			; Are leds enabled? Without this, there were weird display's segments behaviour.
    goto	endleds			; No, get out
    btfsc	leds_s,5		; Is led up on?
    goto	$+3
    bsf		LED_UP			; No
    goto	$+2
    bcf		LED_UP			; Yes
    btfsc	leds_s,4		; Is led down on?
    goto	$+3
    bsf		LED_DWN			; No
    goto	$+2
    bcf		LED_DWN			; Yes
    btfsc	leds_s,3		; Is led light on?
    goto	$+3
    bsf		LED_LIGHT		; No
    goto	$+2
    bcf		LED_LIGHT		; Yes
    btfsc	leds_s,2		; Is led toast on?
    goto	$+3
    bsf		LED_TOASTER		; No
    goto	$+2
    bcf		LED_TOASTER		; Yes
    btfsc	leds_s,1		; Is led time on?
    goto	$+3
    bsf		LED_timer		; No
    goto	$+2
    bcf		LED_timer		; Yes
    btfsc	leds_s,0		; Is led temp on?
    goto    $+3
    bsf		LED_TEMP		; No
    goto	$+2
    bcf		LED_TEMP		; YES
endleds:
    bcf		LEDS_ON			; Here, I turn on the leds anode (transistor Q6)

    ; Test_Keys (k_pressed):
    ;   7       6       5       4       3       2       1       0
    ;   X      up     timer   onoff    light   toast   temp    down

    bsf		STATUS,RP0		; RP0 = 1 goto Bank 1

    ;bcf		OPTION_REG,NOT_RABPU ; Activate pull-up resistors - Clear RABPU flag

    movlw	B'00111100'		; 0 = output - 1 = input
    movwf	TRISA			; Make RA2, RA4, RA3 and RA5 inputs (RA3 is a sensor)
    movlw	B'11110000'		; 0 = output - 1 = input
    movwf	TRISB			; Make RB4, RB5, RB6 and RB7 inputs



	movlw	B'00110100'		; 1 = enable pull-up - 0 disable it.
	movwf	WPUA			; Enable RA2, RA4 and RA5 pull-up internal resistors
    bcf		STATUS,RP0		; RP0 = 0 -> Bank 2
	bsf		STATUS,RP1		; RP1 = 1 -> "   "
    movlw	B'11110000'		; 1 = enable pull-up - 0 disable it.
    movwf	WPUB			; Enable RB4, RB5, RB6 and RB7 pull-up internal resistors
    bsf		STATUS,RP0		; RP0 = 1 -> Back to Bank 1
	bcf		STATUS,RP1		; RP1 = 0 -> "   "
	bsf		OPTION_REG,7	; Set RABPU flag



    bcf		STATUS,RP0		; RP0 = 0 -> Back to Bank 0 (RP1 already = 0)

	;==================================================================================

	;This section has a number of sub-routines to detect a key press, set a flag and turn on a LED
	;This routine detects if a key is pressed for the second
	;time so that a LED can be turned off. It does this by detecting the 
	;key is pressed and looking to see if the "not-pressed" flag has been 
	;set to indicate the key has been released between detections.

	btfsc	KEY_UP			; When KEY_UP pressed, input=0
	goto	$+5				; Not pressed, goto exit (l1)...
	btfss	k_not_pressed,6	; key pressed - look to see if second pressing
	goto	$+3				; (if k_not_pressed,6 is set, key has been released between detections)
	bcf		k_pressed,6		; clear the KEY_UP flag
	incf	temper,F		; increment temperature

	btfsc	KEY_DWN			; When KEY_UP pressed, input=0
	goto	$+5				; Not pressed, goto exit (l1)...
	btfss	k_not_pressed,0	; key pressed - look to see if second pressing
	goto	$+3				; (if k_not_pressed,6 is set, key has been released between detections)
	bcf		k_pressed,0		; clear the KEY_UP flag
	decf	temper,F		; decrement temperature

	;This routine detects if a key is released

	btfss	KEY_UP			; Was the key released? When KEY_UP not pressed, input=1
	goto	$+2				; No, it's still being pressed...
	bsf		k_not_pressed,6	; Key is not pressed, set flag "not pressed"

	btfss	KEY_DWN			; Was the key released? When KEY_UP not pressed, input=1
	goto	$+2				; No, it's still being pressed...
	bsf		k_not_pressed,0	; Key is not pressed, set flag "not pressed"

	;This routine just detects if a key is pressed. 
	;If a key is pressed, it will set the corresponding flag
	;Another routine will detect that the key has been released. 
	;and other routine will deal with doing something will the "key-pressed" flag. 
	;and another routine will deal with turning off the "key-pressed" flag. 
	;clear the flags file in start_up

	btfsc	KEY_UP			; Is KEY_UP pressed, then input=0
	goto	$+3				; No, it's not pressed, exit...
	bsf		k_pressed,6		; Yes, it's pressed, set flag
	bcf		k_not_pressed,6 ; Clear the flag which indicates the key as not pressed.

	btfsc	KEY_DWN			; Is KEY_UP pressed, then input=0
	goto	$+3				; No, it's not pressed, exit...
	bsf		k_pressed,0		; Yes, it's pressed, set flag
	bcf		k_not_pressed,0 ; Clear the flag which indicates the key as not pressed.

	;==================================================================================

    bsf		STATUS,RP0		; RP0 = 1 -> Goto Bank 1 (RP1 already = 0)
    bcf		WPUA,7
    bcf		WPUB,7
    bcf		OPTION_REG,7	; Set RABPU flag
    movlw	B'00001000'		;
    movwf	TRISA			; Make A0,A1,A2,A4,A5,A6 and A7 outputs.
    movlw	B'00000000'		;
    movwf	TRISB			; Make all PORTB output.
    bcf		STATUS,RP0		; RP0 = 0 -> Back to Bank 0 (RP1 already = 0)

    retlw    00

;/****************************************************************************
;* DESCRIPTION: Main Program
;*****************************************************************************/ 

Main:
	movf	temper,W		; Put temperature in W
	call	bin2bcd			; Decode Temperature to bcd
	movf	units,W			; copy units to W
	call	decdisp_a		; convert
	bsf		LED_CMN			; Leds off
	movwf	PORTA			; put it in PORTA
	movf	units,W			; copy units to W
	call	decdisp_b		; convert
	movwf	PORTB			; put it in PORTA
	bcf		DISP1			; display 1 on
	call	DelayAA			; delay
	call	H_keeping		; delay 2 ms
	movf	tens,W			; copy tens to W
	call	decdisp_a		; convert
	bsf		DISP1			; display 1 off
	movwf	PORTA			; put it in PORTA
	movf	tens,W			; copy tens to W
	call	decdisp_b		; convert
	movwf	PORTB			; put it in PORTA
	bcf		DISP2			; display 2 on
	call	DelayAA			; delay
	call	H_keeping		; delay 2 ms
	movf	hundreds,W		; copy hundeds to W
	call	decdisp_a		; convert
	bsf		DISP2			; display 2 off
	movwf	PORTA			; put it in PORTA
	movf	hundreds,W		; copy hundeds to W
	call	decdisp_b		; convert
	movwf	PORTB			; put it in PORTA
	bcf		DISP3			; display 3 on
	call	DelayAA			; delay
	call	H_keeping		; delay 2 ms
	bsf		DISP3			; display 3 off

	; Leds code:
	bsf		LEDS_ON			; Flag leds on
	bcf		LED_CMN			; Leds on
	call	H_keeping		; delay 2 ms

	goto	Main			; Back to main loop

	END



